// BePrinter 0.030 by Earl Colby Pottinger. 
// Partly based on code found in the Beos CDROM 
// Partly based on instructions found in "Programming the Be Operating System" By Dan Parks Sydow. 
// Additional help by Larry Baydak in debugging and coding/commenting style. 

#include "BePrinter_Includes.h"
#include "BePrinter_Menus.h"
#include "BePrinter_Defaults.h"

//Constants needed. 
const char *APP_SIGNATURE="application/x-vnd.BePrinter"; // Application's signature.

//Variables needed to control printer.
char PrintCPI[99];
char PrintQuality[99], PrintLQ[99];
char PrintSpacing[99], PrintSpace[99];
char PaperTray[99], PaperSource[99];
char PrintOrientation[99],PrintOrient[99];
char PrintWrap[99], PrintWRAP[99];
char PrintDirection[99], PrintDir[99];

float PortraitX, PortraitY;   // Width & Height in inches.
float LandscapeX, LandscapeY; // Width & Height in inches
float X, Y; // Speeds drawing to screen. 
float CPI; // Characters per inch.
float LPI; // Lines per inch.

long PaperX; // Width in characters.
long PaperY; // Height in Lines.

// File and Printer buffers.
FILE *PrinterPort;         // Handle for printer port.
long FileLenght=0;         // Number of bytes in file input buffer.
char FileBuffer[999999];   // Buffers file to print. (Temp solution).
long LineLenght=0;         // Number of bytes in output buffer.
char LineBuffer[99999];    // Buffers for one line of graphic output.
char TextBuffer[99];       // Text buffer to convert number variables.

// Objects needed.
BFile      FileObject;     // BFile object
BFilePanel *OpenFilePanel; // Define object to create open panels from.
BAlert     *alert;         // Create a pointer (*alert) to an alert window
 long       result=0;      // Returned value from the BAlert::Go() member function.

// BePrinterView.h
class BePrinterView : public BView {                   // Define the class object BePrinterView.
 public:       BePrinterView(BRect frame, char *name); // Constructor for a view inside a window.
  virtual void AttachedToWindow();                     // Called by the OS when 'aWindow->AddChild(aView)' used.
  virtual void Draw(BRect updateRect);                 // Called by the OS when 'aWindow->Show()' used.
} ;

// BePrinterView.cpp
BePrinterView::BePrinterView(BRect rect, char *name) // Define 'BePrinterView' constructor call.
 : BView(rect, name, B_FOLLOW_ALL, B_WILL_DRAW) {    //
} ;

void BePrinterView::AttachedToWindow() { // Sets the character of the view when a view ia attached to a window.
 SetFont(be_plain_font); // Set font outline to bold.
 SetFontSize(12);         // Set font size.
} ;

void BePrinterView::Draw(BRect updateRect) { // Draws in view when show is called.
 X=0.0; Y=0.0; // Move pen to top of window.
 X=20.0; Y=Y+20.0; MovePenTo(BPoint(X,Y)); sprintf(TextBuffer,"Printer Name: <%s>",PrintName); DrawString(TextBuffer); // Name of printer or family.
 X=20.0; Y=Y+20.0; MovePenTo(BPoint(X,Y)); sprintf(TextBuffer,"Printer Mode: <%s>",PrintMode); DrawString(TextBuffer); // Choice of Text or Graphic mode.
 X=20.0; Y=Y+20.0; MovePenTo(BPoint(X,Y)); sprintf(TextBuffer,"Filtering(Y/N): <%s>",FilterCodes); DrawString(TextBuffer); // Filter out all control characters?
 X=20.0; Y=Y+20.0; MovePenTo(BPoint(X,Y)); sprintf(TextBuffer,"Top margin: %ld",TopBlankLines); DrawString(TextBuffer); // Number of lines from Top of Page.
 X=20.0; Y=Y+20.0; MovePenTo(BPoint(X,Y)); sprintf(TextBuffer,"Bottom margin: %ld",BottomBlankLines); DrawString(TextBuffer); // Number of lines from Bottom of Page.
 X=20.0; Y=Y+20.0; MovePenTo(BPoint(X,Y)); sprintf(TextBuffer,"Tab Size(chars): %ld",TabSize); DrawString(TextBuffer); // Size of tab steps. 
 X=20.0; Y=Y+20.0; MovePenTo(BPoint(X,Y)); sprintf(TextBuffer,"CPI: %f",CPI); DrawString(TextBuffer); // Characters per inch.
 X=20.0; Y=Y+20.0; MovePenTo(BPoint(X,Y)); sprintf(TextBuffer,"Pages per Sheet: %ld",PagesPerSheet); DrawString(TextBuffer); //
 X=20.0; Y=Y+20.0; MovePenTo(BPoint(X,Y)); sprintf(TextBuffer,"Columns per Page: %ld",ColumnsPerPage); DrawString(TextBuffer); //
 X=20.0; Y=Y+20.0; MovePenTo(BPoint(X,Y)); sprintf(TextBuffer,"Long Line Mode: <%s>",LineMode); DrawString(TextBuffer); // Truncate, LineWrap or WordWrap long lines of text.
 X=20.0; Y=Y+20.0; MovePenTo(BPoint(X,Y)); sprintf(TextBuffer,"Codes to Reset Printer: <%s>",ResetPrinter); DrawString(TextBuffer); // Resets printer, sent at start of first printing.
 X=20.0; Y=Y+20.0; MovePenTo(BPoint(X,Y)); sprintf(TextBuffer,"Codes to Load_Paper: <%s>",LoadPaper); DrawString(TextBuffer); // Loads paper before printing.
 X=20.0; Y=Y+20.0; MovePenTo(BPoint(X,Y)); sprintf(TextBuffer,"Codes to Eject_Paper: <%s>",EjectPaper); DrawString(TextBuffer); // Ejects present sheet, may load another sheet.
 X=20.0; Y=Y+20.0; MovePenTo(BPoint(X,Y)); sprintf(TextBuffer,"Paper tray to use: <%s>",PaperSource); DrawString(TextBuffer); // Where to load paper from.
 X=20.0; Y=Y+20.0; MovePenTo(BPoint(X,Y)); sprintf(TextBuffer,"Print Direction(s): <%s>",PrintDir); DrawString(TextBuffer); //
 X=20.0; Y=Y+20.0; MovePenTo(BPoint(X,Y)); sprintf(TextBuffer,"Printer's Linewrapping: <%s>",PrintWRAP); DrawString(TextBuffer); // Printer setting for line wrapping.
 X=20.0; Y=Y+20.0; MovePenTo(BPoint(X,Y)); sprintf(TextBuffer,"Print Data Path: <%s>",PrinterPath); DrawString(TextBuffer); // Where data is sent after processing.
 X=20.0; Y=Y+20.0; MovePenTo(BPoint(X,Y)); sprintf(TextBuffer,"Paper Orientation: <%s>",PrintOrient); DrawString(TextBuffer); // 
 X=20.0; Y=Y+20.0; MovePenTo(BPoint(X,Y)); sprintf(TextBuffer,"Proportional Spacing: <%s>",PrintSpace); DrawString(TextBuffer); // Proportional or Fixed spacing
 X=20.0; Y=Y+20.0; MovePenTo(BPoint(X,Y)); sprintf(TextBuffer,"Print Quality: <%s>",PrintLQ); DrawString(TextBuffer); // Print Quality.
}

// BePrinterWindow.h
class BePrinterWindow : public BWindow {           // Define the class object BePrinterWindow.
 public:       BePrinterWindow(BRect frame);       // Constructor for a window thread.
  virtual bool QuitRequested();                    // Add function is close box is clicked.
  virtual void MessageReceived(BMessage *message); // Setup to recieve menu & windows messages.
 private:
  BMenuBar     *menubar;                           // Define object to hold all menu structures.
 
} ;

// BePrinterWindow.cpp
BePrinterWindow::BePrinterWindow(BRect frame) // Define 'BePrinterWindow' constructor call.
 : BWindow(frame,                   // Frame area of the window
           "BePrinter Window",      // Name of the window, appears in the Tab on top of the window.
           B_TITLED_WINDOW,         // Type of window (Look & Feel)
           B_NOT_RESIZABLE )        // Behavior of the window (resize knob, etc)
{
 BRect rMenu;                       // Set the Window Bpoints into the rectangle object(aRect) to define a MenuBar.
 BMenu *menu;                       // 
 rMenu = Bounds();                  // Find active area of view.
 menubar = new BMenuBar(rMenu,      // Invoke a new dynamically allocated menubar object inside the window
          "printer menu_bar");      // by calling the constructor. Plus supply a ID name for the menu.
 AddChild(menubar);                 // Add menubar to window created.
 
 menu = new BMenu("File"); // Adds File menu to menu bar.
 menu->AddItem(new BMenuItem("New",       new BMessage(MENU_FILE_NEW),   'N')); // Not used.
 menu->AddItem(new BMenuItem("Open",      new BMessage(MENU_FILE_OPEN),  'O')); // Not used.
 menu->AddItem(new BMenuItem("Close",     new BMessage(MENU_FILE_CLOSE), 'W')); // Not used.
 menu->AddItem(new BMenuItem("Save",      new BMessage(MENU_FILE_SAVE),  'S')); // Not used.
 menu->AddItem(new BMenuItem("Save as",   new BMessage(MENU_FILE_SAVEAS)    )); // Not used.
 menu->AddSeparatorItem();
 menu->AddItem(new BMenuItem("Page Setup",new BMessage(MENU_FILE_SETUP)     )); // Setup page margins.
 menu->AddItem(new BMenuItem("Print",     new BMessage(MENU_FILE_PRINT), 'P')); // Print a flie or clipping.
 menu->AddItem(new BMenuItem("Quit",      new BMessage(MENU_FILE_QUIT),  'Q')); // Exit program.
 menubar->AddItem(menu);

 menu = new BMenu("Printers"); // Adds Printer menu to menu bar
 menu->AddItem(new BMenuItem("All Raw Data",  new BMessage(MENU_PRINTER_RAW)    )); // Raw data, uses no filtering for control codes.
 menu->AddItem(new BMenuItem("Pure ASCII",    new BMessage(MENU_PRINTER_ASCII)  )); // Pure ASCII, filters control codes.
 menu->AddItem(new BMenuItem("Generic ASCII", new BMessage(MENU_PRINTER_GENERIC))); // Generic ASCII, translates basic control codes.
 menu->AddSeparatorItem();
 menu->AddItem(new BMenuItem("HP Deskjet Series",new BMessage(MENU_PRINTER_HP_INKJET) )); // HP DeskJets.
 menu->AddItem(new BMenuItem("Epson Stylus",     new BMessage(MENU_PRINTER_EPSON_INK) )); // Epson ESC/P2.
 menu->AddItem(new BMenuItem("Epson Dot Matrix", new BMessage(MENU_PRINTER_EPSON_DOT) )); // Epson Dot Matrix.
 menu->AddItem(new BMenuItem("IBM Proprinter",   new BMessage(MENU_PRINTER_PROPRINTER))); // IBM Proprinter III.
 menu->AddSeparatorItem();
 menu->AddItem(new BMenuItem("Parallel Port 1", new BMessage(MENU_PRINTER_PAR1))); // Printer's Parallel port 1 interface.
 menu->AddItem(new BMenuItem("Parallel Port 2", new BMessage(MENU_PRINTER_PAR2))); // Printer's Parallel port 2 interface.
 menu->AddItem(new BMenuItem("Serial Port 1",   new BMessage(MENU_PRINTER_SER1))); // Printer's Serial port 1 interface.
 menu->AddItem(new BMenuItem("Serial Port 2",   new BMessage(MENU_PRINTER_SER2))); // Printer's Serial port 2 interface.
 menu->AddItem(new BMenuItem("Printer File",    new BMessage(MENU_PRINTER_PRNT))); // Printer's Print to File.
 menubar->AddItem(menu);
 
 menu = new BMenu("Print Modes"); // Adds Print mode menu to menu bar
 menu->AddItem(new BMenuItem("Text mode",    new BMessage(MENU_MODE_TEXT)   )); // Print using a printer's text mode, does not support pictures.
 menu->AddItem(new BMenuItem("Graphic mode", new BMessage(MENU_MODE_GRAPHIC))); // Print using a printer's graphic mode, does support pictures.
 menu->AddSeparatorItem();
 menu->AddItem(new BMenuItem("Normal Text (10 CPI)",     new BMessage(MENU_MODE_NORMAL)    )); // Prints text in normal character size.
 menu->AddItem(new BMenuItem("Compressed Text (12 CPI)", new BMessage(MENU_MODE_COMPRESS12))); // Prints text in (12 CPI) size.
 menu->AddItem(new BMenuItem("Compressed Text (15 CPI)", new BMessage(MENU_MODE_COMPRESS15))); // Prints text in (15 CPI) size.
 menu->AddItem(new BMenuItem("Compressed Text (17 CPI)", new BMessage(MENU_MODE_COMPRESS17))); // Prints text in (17 CPI) size.
 menu->AddItem(new BMenuItem("Compressed Text (20 CPI)", new BMessage(MENU_MODE_COMPRESS20))); // Prints text in (20 CPI) size.
 menu->AddSeparatorItem();
 menu->AddItem(new BMenuItem("Draft Mode",           new BMessage(MENU_MODE_DRAFT))); // Prints text in Draft Mode.
 menu->AddItem(new BMenuItem("Near Letter Quality",  new BMessage(MENU_MODE_NLQ)  )); // Prints text in Near Letter Quality Mode.
 menu->AddItem(new BMenuItem("Letter Quality",       new BMessage(MENU_MODE_LQ)   )); // Prints text in Letter Quality Mode.
 menu->AddSeparatorItem();
 menu->AddItem(new BMenuItem("Proportional Spacing", new BMessage(MENU_MODE_PS)   )); // Prints text in Proportional Spacing Mode.
 menu->AddItem(new BMenuItem("Fixed Spacing",        new BMessage(MENU_MODE_PS)   )); // Prints text in Proportional Spacing Mode.
 menu->AddSeparatorItem();
 menu->AddItem(new BMenuItem("1 Page per Sheet",  new BMessage(MENU_MODE_1PAGE),'1')); // Print one page on one sheet of paper.
 menu->AddItem(new BMenuItem("2 Pages per Sheet", new BMessage(MENU_MODE_2PAGE),'2')); // Print two pages on one sheet of paper.
 menu->AddItem(new BMenuItem("4 Pages per Sheet", new BMessage(MENU_MODE_4PAGE),'4')); // Print four pages on one sheet of paper.
 menu->AddItem(new BMenuItem("8 Pages per Sheet", new BMessage(MENU_MODE_8PAGE),'8')); // Print eight page on one sheet of paper.
 menu->AddSeparatorItem();
 menu->AddItem(new BMenuItem("1 Column per Page",  new BMessage(MENU_MODE_1COLUMN))); // Prints one column on a page.
 menu->AddItem(new BMenuItem("2 Columns per Page", new BMessage(MENU_MODE_2COLUMN))); // Prints two columns on a page.
 menu->AddItem(new BMenuItem("3 Columns per Page", new BMessage(MENU_MODE_3COLUMN))); // Prints three columns on a page.
 menu->AddItem(new BMenuItem("4 Columns per Page", new BMessage(MENU_MODE_4COLUMN))); // Prints four columns on a page.
 menu->AddSeparatorItem();
 menu->AddItem(new BMenuItem("Long Lines Truncated",    new BMessage(MENU_MODE_LINE_T))); // Truncate extra long lines of text.
 menu->AddItem(new BMenuItem("Long Lines LineWrapped",  new BMessage(MENU_MODE_LINE_L))); // LineWrap extra long lines of text.
 menu->AddItem(new BMenuItem("Long Lines Word Wrapped", new BMessage(MENU_MODE_LINE_W))); // WordWrap extra long lines of text.
 menubar->AddItem(menu);

 menu = new BMenu("Command Printer"); // Adds Command menu to menu bar
 menu->AddItem(new BMenuItem("Reset Printer",   new BMessage(MENU_CMMD_RESET) )); // Reset printer.
 menu->AddItem(new BMenuItem("Print Test Page", new BMessage(MENU_CMMD_TEST)  )); // Print a test page.
 menu->AddItem(new BMenuItem("Load a sheet",    new BMessage(MENU_CMMD_LOAD)  )); // Load a sheet of paper into printer.
 menu->AddItem(new BMenuItem("Eject a sheet",   new BMessage(MENU_CMMD_EJECT) )); // Eject a sheet of paper from printer.
 menu->AddSeparatorItem();
 menu->AddItem(new BMenuItem("Select Exec paper",   new BMessage(MENU_CMMD_EXEC)  )); // Selects the Exec sized paper tray.
 menu->AddItem(new BMenuItem("Select Letter paper", new BMessage(MENU_CMMD_LETTER))); // Selects the Letter sized paper tray.
 menu->AddItem(new BMenuItem("Select Legal paper",  new BMessage(MENU_CMMD_LEGAL) )); // Selects the Legal sized paper tray.
 menu->AddItem(new BMenuItem("Select A4 paper",     new BMessage(MENU_CMMD_A4)    )); // Selects the A4 sized paper tray.
 menu->AddSeparatorItem();
 menu->AddItem(new BMenuItem("Select Portrait",  new BMessage(MENU_CMMD_PORTRAIT) )); // Selects the Exec sized paper tray.
 menu->AddItem(new BMenuItem("Select Landscape", new BMessage(MENU_CMMD_LANDSCAPE))); // Selects the Letter sized paper tray.
 menu->AddSeparatorItem();
 menu->AddItem(new BMenuItem("Printer's Linewrap on",  new BMessage(MENU_CMMD_LINEWRAP))); // Selects the Legal sized paper tray.
 menu->AddItem(new BMenuItem("Printer's Linewrap off", new BMessage(MENU_CMMD_NOWRAP)  )); // Selects the A4 sized paper tray.
 menu->AddSeparatorItem();
 menu->AddItem(new BMenuItem("Unidirectional Printing", new BMessage(MENU_CMMD_UNIDIR))); // Selects the Legal sized paper tray.
 menu->AddItem(new BMenuItem("Bidirectional Printing",  new BMessage(MENU_CMMD_BIDIR) )); // Selects the A4 sized paper tray.
 menubar->AddItem(menu);
 Show();
} ;

bool BePrinterWindow::QuitRequested() { // Close button on window pressed. A Blooper call to Bwindows. 
 be_app->PostMessage(B_QUIT_REQUESTED); // Send a message to the global variable 'be_app' to tell the program to quit.
 return(true);                          // Tell the system (Blooper) to go ahead and close window.
} ;

// BePrinterWindow::MessageReceived.  Called when a message is received by our application.
void BePrinterWindow::MessageReceived(BMessage *message) {
 switch(message->what) {
  case MENU_FILE_NEW:   {
   alert = new BAlert("Printer Alert","This function is not properly supported yet.","OK"); result = alert->Go(); // Displays alert with warning.
   break; } ;
  case MENU_FILE_OPEN:  {
   alert = new BAlert("Printer Alert","This function is not properly supported yet.","OK"); result = alert->Go(); // Displays alert with warning.
   break; } ; // Not used yet.
  case MENU_FILE_CLOSE: {
   alert = new BAlert("Printer Alert","This function is not properly supported yet.","OK"); result = alert->Go(); // Displays alert with warning.
   break; } ; // Not used yet.
  case MENU_FILE_SAVE:  {
   alert = new BAlert("Printer Alert","This function is not properly supported yet.","OK"); result = alert->Go(); // Displays alert with warning.
   break; } ; // Not used yet.
  case MENU_FILE_SAVEAS:  {
   alert = new BAlert("Printer Alert","This function is not properly supported yet.","OK"); result = alert->Go(); // Displays alert with warning.
   break; } ; // Not used yet.
  case MENU_FILE_SETUP: {
   alert = new BAlert("Printer Alert","This function is not properly supported yet.","OK"); result = alert->Go(); // Displays alert with warning.
   break; } ; // Setup page margins.

  case MENU_FILE_PRINT: { // Print a file or clipping to the output port.
   OpenFilePanel = new BFilePanel(B_OPEN_PANEL); // Create the open a file panel.
   OpenFilePanel->Show(); // Display open a file panel, get file name to print.   
   break; } ;             // End of case

  case MENU_FILE_QUIT: {  // Exit the program.
  if (PrinterPort!=0) {   // Test if printer port is open.
   fclose(PrinterPort);   // If it is, close it.
   PrinterPort=NULL ; } ; // Clear the handle to show it is closed.
  be_app->PostMessage(B_QUIT_REQUESTED);
  break; } ;              // Exit program, sends a message to BApplication.

  #include "RAW_ASCII.cpp"     // Settings for the transfering the file unchanged to printer.
  #include "PURE_ASCII.cpp"    // Settings for printing the file with filtering/processing EOL markers.
  #include "GENERIC_ASCII.cpp" // Settings to filter out all control codes, and process EOL markers.
  #include "HP_INKJET.cpp"     // Settings for the HP DeskJet (PCL3) printers.
  #include "EPSON_INK.cpp"     // Settings for the Epson Stylus ESC/P2 printers
  #include "EPSON_DOT.cpp"     // Settings for the Epson Dot Martix printers.
  #include "PROPRINTER.cpp"    // Settings for the IBM ProPrinter III.

  case MENU_PRINTER_PAR1: { PrinterPath = "/dev/parallel/parallel1"; Show(); break; } ; // Printer's Parallel port 1 interface.  Redraw window with the view attached.
  case MENU_PRINTER_PAR2: { PrinterPath = "/dev/parallel/parallel2"; Show(); break; } ; // Printer's Parallel port 2 interface.  Redraw window with the view attached.
  case MENU_PRINTER_SER1: { PrinterPath = "/dev/serial/serial1";     Show(); break; } ; // Printer's Serial port 1 interface.  Redraw window with the view attached.
  case MENU_PRINTER_SER2: { PrinterPath = "/dev/serial/serial2";     Show(); break; } ; // Printer's Serial port 2 interface.  Redraw window with the view attached.
  case MENU_PRINTER_PRNT: { PrinterPath = "PrintFile";               Show(); break; } ; // Printer's Print to File.  Redraw window with the view attached.

  case MENU_MODE_TEXT:    {
   PrintMode="Text";     // Use printer in it's text mode.
   Show(); break; } ;    // Redraw window with the view attached.   
  case MENU_MODE_GRAPHIC: {
   PrintMode="Graphic"; // Use printer using graphic mode.
   alert = new BAlert("Printer Alert","This function is not properly supported yet.","OK"); result = alert->Go(); // Displays alert with warning.
   Show(); break; } ;   // Redraw window with the view attached.  

  case MENU_MODE_DRAFT: {
   strcpy(PrintQuality,DraftMode); strcpy(PrintLQ,"Draft mode"); // Use printer using draft mode.
   Show(); break; } ; // Redraw window with the view attached.  
  case MENU_MODE_NLQ: {
   strcpy(PrintQuality,NLQ_Mode); strcpy(PrintLQ,"Near Letter Quality mode"); // Use printer using Near Letter Quality mode.
   Show(); break; } ; // Redraw window with the view attached.  
  case MENU_MODE_LQ: {
   strcpy(PrintQuality,LQ_Mode); strcpy(PrintLQ,"Letter Quality mode"); // Use printer using Letter Quality mode.
   Show(); break; } ; // Redraw window with the view attached.  

  case MENU_MODE_PS: {
   strcpy(PrintSpacing,PS_ModeOn); strcpy(PrintSpace,"Proportional Spacing"); // Print using Proportional spacing.
   Show(); break; } ; // Redraw window with the view attached.  
  case MENU_MODE_FIXED: {
   strcpy(PrintSpacing,PS_ModeOff); strcpy(PrintSpace,"Fixed Spacing"); // Print using Fixed spacing.
   Show(); break; } ; // Redraw window with the view attached.  

  case MENU_MODE_NORMAL: {
   strcpy(PrintCPI,CPI10); CPI=CPI_10; // Print using normal size charaters.
   Show(); break; } ;                  // Redraw window with the view attached.
  case MENU_MODE_COMPRESS12: {
   strcpy(PrintCPI,CPI12); CPI=CPI_12; // Print using compressed charaters.
   Show(); break; } ;                  // Redraw window with the view attached.
  case MENU_MODE_COMPRESS15: {
   strcpy(PrintCPI,CPI15); CPI=CPI_15; // Print using compressed charaters.
   Show(); break; } ;                  // Redraw window with the view attached.
  case MENU_MODE_COMPRESS17: {
   strcpy(PrintCPI,CPI17); CPI=CPI_17; // Print using compressed charaters.
   Show(); break; } ;                  // Redraw window with the view attached.
  case MENU_MODE_COMPRESS20: {
   strcpy(PrintCPI,CPI20); CPI=CPI_20; // Print using compressed charaters.
   Show(); break; } ;                  // Redraw window with the view attached.

  case MENU_MODE_1PAGE: {
   PagesPerSheet=1;   // Print one page on one sheet of paper.
   alert = new BAlert("Printer Alert","This function is not properly supported yet.","OK"); result = alert->Go(); // Displays alert with warning.
   Show(); break; } ; // Redraw window with the view attached.
  case MENU_MODE_2PAGE: {
   PagesPerSheet=2;   // Print two pages on one sheet of paper.
   alert = new BAlert("Printer Alert","This function is not properly supported yet.","OK"); result = alert->Go(); // Displays alert with warning.
   Show(); break; } ; // Redraw window with the view attached.
  case MENU_MODE_4PAGE: {
   PagesPerSheet=4;   // Print four pages on one sheet of paper.
   alert = new BAlert("Printer Alert","This function is not properly supported yet.","OK"); result = alert->Go(); // Displays alert with warning.
   Show(); break; } ; // Redraw window with the view attached.
  case MENU_MODE_8PAGE: {
   PagesPerSheet=8;   // Print eight page on one sheet of paper.
   alert = new BAlert("Printer Alert","This function is not properly supported yet.","OK"); result = alert->Go(); // Displays alert with warning.
   Show(); break; } ; // Redraw window with the view attached.

  case MENU_MODE_1COLUMN: {
   ColumnsPerPage=1;  // Prints one column on a page.
   alert = new BAlert("Printer Alert","This function is not properly supported yet.","OK"); result = alert->Go(); // Displays alert with warning.
   Show(); break; } ; // Redraw window with the view attached.
  case MENU_MODE_2COLUMN: {
   ColumnsPerPage=2;  // Prints two columns on a page.
   alert = new BAlert("Printer Alert","This function is not properly supported yet.","OK"); result = alert->Go(); // Displays alert with warning.
   Show(); break; } ; // Redraw window with the view attached.
  case MENU_MODE_3COLUMN: {
   ColumnsPerPage=3;  // Prints three columns on a page.
   alert = new BAlert("Printer Alert","This function is not properly supported yet.","OK"); result = alert->Go(); // Displays alert with warning.
   Show(); break; } ; // Redraw window with the view attached.
  case MENU_MODE_4COLUMN: {
   ColumnsPerPage=4;  // Prints four columns on a page.
   alert = new BAlert("Printer Alert","This function is not properly supported yet.","OK"); result = alert->Go(); // Displays alert with warning.
   Show(); break; } ; // Redraw window with the view attached.


  case MENU_MODE_LINE_T: {
   LineMode="Truncated"; // Truncate extra long lines of text.
   alert = new BAlert("Printer Alert","This function is not properly supported yet.","OK"); result = alert->Go(); // Displays alert with warning.
   Show(); break; } ;    // Redraw window with the view attached.  
  case MENU_MODE_LINE_L: {
   LineMode="LineWrap"; // LineWrap extra long lines of text.
   alert = new BAlert("Printer Alert","This function is not properly supported yet.","OK"); result = alert->Go(); // Displays alert with warning.
   Show(); break; } ;   // Redraw window with the view attached.
  case MENU_MODE_LINE_W: {
   LineMode="WordWarp"; // WordWrap extra long lines of text.
   alert = new BAlert("Printer Alert","This function is not properly supported yet.","OK"); result = alert->Go(); // Displays alert with warning.
   Show(); break; } ;   // Redraw window with the view attached.
  
  case MENU_CMMD_RESET: {
   alert = new BAlert("Printer Alert","This function is not properly supported yet.","OK"); result = alert->Go(); // Displays alert with warning.
   break; } ; // Reset printer.
  case MENU_CMMD_TEST:  {
   alert = new BAlert("Printer Alert","This function is not properly supported yet.","OK"); result = alert->Go(); // Displays alert with warning.
   break; } ; // Print a test page.
  case MENU_CMMD_LOAD:  {
   alert = new BAlert("Printer Alert","This function is not properly supported yet.","OK"); result = alert->Go(); // Displays alert with warning.
   break; } ; // Load a sheet of paper into printer.
  case MENU_CMMD_EJECT: {
   alert = new BAlert("Printer Alert","This function is not properly supported yet.","OK"); result = alert->Go(); // Displays alert with warning.
   break; } ; // Eject a sheet of paper from printer.
  
  case MENU_CMMD_EXEC:   {
   strcpy(PaperTray,SelectExec); strcpy(PaperSource,"Executive"); // Selects the Exec sized paper tray.
   PortraitX=PortraitExecX;   PortraitY=PortraitExecY;   // Width & Height in inches.
   LandscapeX=LandscapeExecX; LandscapeY=LandscapeExecY; // Width & Height in inches.
   Show(); break; } ; // Redraw window with the view attached.
  case MENU_CMMD_LETTER: {
   strcpy(PaperTray,SelectLetter); strcpy(PaperSource,"Letter"); // Selects the Letter sized paper tray.
   PortraitX=PortraitLetterX;   PortraitY=PortraitLetterY;   // Width & Height in inches.
   LandscapeX=LandscapeLetterX; LandscapeY=LandscapeLetterY; // Width & Height in inches.
   Show(); break; } ; // Redraw window with the view attached.
  case MENU_CMMD_LEGAL: {
   strcpy(PaperTray,SelectLegal); strcpy(PaperSource,"Legal"); // Selects the Legal sized paper tray.
   PortraitX=PortraitLegalX;   PortraitY=PortraitLegalY;   // Width & Height in inches.
   LandscapeX=LandscapeLegalX; LandscapeY=LandscapeLegalY; // Width & Height in inches.
   Show(); break; } ; // Redraw window with the view attached.
  case MENU_CMMD_A4: {
   strcpy(PaperTray,SelectA4); strcpy(PaperSource,"A4"); // Selects the A4 sized paper tray.
   PortraitX=PortraitA4X;   PortraitY=PortraitA4Y;   // Width & Height in inches.
   LandscapeX=LandscapeA4X; LandscapeY=LandscapeA4Y; // Width & Height in inches.
   Show(); break; } ; // Redraw window with the view attached.

  case MENU_CMMD_PORTRAIT:  { 
   strcpy(PrintOrientation,PortraitPage); strcpy(PrintOrient,"Portrait");   // Selects printing in Portrait mode.
   Show(); break; } ; // Redraw window with the view attached.
  case MENU_CMMD_LANDSCAPE: {
   strcpy(PrintOrientation,LandscapePage); strcpy(PrintOrient,"Landscape"); // Selects printing in Landscape mode.
   Show(); break; } ; // Redraw window with the view attached.

  case MENU_CMMD_LINEWRAP: {
   strcpy(PrintWrap,EnableEOLWrap); strcpy(PrintWRAP,"Printer LineWrapping ON");   // Turns off the printer's EOL linewrapping.
   Show(); break; } ; // Redraw window with the view attached.
  case MENU_CMMD_NOWRAP:   {
   strcpy(PrintWrap,DisableEOLWrap); strcpy(PrintWRAP,"Printer LineWrapping OFF"); // Turns off the printer's EOL linewrapping.
   Show(); break; } ; // Redraw window with the view attached.

  case MENU_CMMD_UNIDIR: {
   strcpy(PrintDirection,Unidirectional); strcpy(PrintDir,"Unidirectional Printing"); // Sets the printer to print left to right only.
   Show(); break; } ; // Redraw window with the view attached.
  case MENU_CMMD_BIDIR:  {
   strcpy(PrintDirection,Bidirectional); strcpy(PrintDir,"Bidirectional Printing");  // Sets the printer to print in both directions.
   Show(); break; } ; // Redraw window with the view attached.

  default:              { BWindow::MessageReceived(message); break; } ;
} ; }

// BePrinterWorld.h
class BePrinterApplication : public BApplication { // Define the class object BePrinterApplication.
 public:       BePrinterApplication();             // Constructor for an application thread.
  virtual void MessageReceived(BMessage *message); // ??????????????
  virtual void RefsReceived(BMessage *message);    // Recieve messages from file panel.
} ;

// BePrinterWorld.cpp  The main() function's only real job in a basic BeOS application is to create the BApplication object and run it.
int main(int, char**) {
 BePrinterApplication myApplication; // Create an Be_app instance?
 myApplication.Run();                // Run the app thread.  Start to process messages.
 return(0);                          // Return 0 for no errors.
} ;

BePrinterApplication::BePrinterApplication() : BApplication(APP_SIGNATURE) { // Define 'BePrinterApplication' constructor call. 
 BePrinterWindow *aWindow;                // Create a pointer(*aWindow) to an object(BePrinterWindow).
 BePrinterView   *aView;                  // Create a pointer(*aView) to an object(BePrinterView).
 BRect           aRect;                   // Set up a rectangle object(aRect). Size of Window to open.

//Sets final defaults needed. 
LPI=6.0; // Lines per inch.
strcpy(PrintCPI,CPI10);               // Set default to 10 Characters per inch.
CPI=CPI_10;
strcpy(PrintQuality,DraftMode);       // Set default to draft mode. 
strcpy(PrintLQ,"Draft mode");
strcpy(PrintSpacing,PS_ModeOff);      // Set default spacing.
strcpy(PrintSpace,"Fixed Spacing");
strcpy(PrintWrap,DisableEOLWrap);     // Set default EOL wrapping.
strcpy(PrintWRAP,"Printer LineWrapping OFF");
strcpy(PrintDirection,Bidirectional); // Set default print direction.
strcpy(PrintDir,"Bidirectional Printing");
strcpy(PaperTray,SelectLetter);       // Set default Letter paper tray.
strcpy(PaperSource,"Letter");
strcpy(PrintOrientation,PortraitPage); // Set for portrait alignment.
strcpy(PrintOrient,"Portrait");
PortraitX=PortraitLetterX;             // Set default Width in inches.
PortraitY=PortraitLetterY;             // Set default Height in inches.

// Create a window. 
 aRect.Set(32, 32, 600, 600);             // Set the Screen Bpoints into the rectangle object(aRect) to define a window.
 aWindow = new BePrinterWindow(aRect);    // Invoke a new dynamically allocated window object by calling the constructor.
 aWindow->Show();                         // Make window visible.

// Create a view.
 aRect.Set(0, 19, 599, 599);              // Set the Window Bpoints into the rectangle object(aRect) to define a view.
 aView = new BePrinterView(aRect,         // Invoke a new dynamically allocated view object inside the window
         "BePrinter_View_ID_Name" ) ;     // by calling the constructor. Plus supply a ID name for the view.
 aWindow->AddChild(aView);                // Add view to window.
 aWindow->Show();                         // Redraw window with the view attached.

} ;

// BePrinterApp::MessageReceived.  Handle incoming messages. 
void BePrinterApplication::MessageReceived(BMessage *message) { // ???????????????
 switch(message->what) {                                        // ???????????????
  default: { BApplication::MessageReceived(message); break; }   // ???????????????
} ; }

 void StartPrinting(); 
 
// BePrinterApp::RefsReceived.  Handle a refs received message.
// The message specifies
// where to get it (see BFilePanel in the Storage Kit chapter of the Be Book).
void BePrinterApplication::RefsReceived(BMessage *message) {
 entry_ref  ref;        // The entry_ref to open, For the directory to save into.
 status_t   err;        // The error/return code
 int32      ref_num;    // The index into the ref list
 off_t      length;     // Length of file.

// Open printer port.
 PrinterPort=fopen(PrinterPath,"w");      // Open file handle to printer.
 if (PrinterPort==0) {                    // Test if port opening failed.
  alert = new BAlert("PrinterAlert",      // If yes, then display Alert box.  Alert Title.
                     "Port failed",       //could not be opened, Continue program?",
                     "OK",                // Button #0 text. Do not quit program.
                     "Quit");             // Button #1 text. Do quit program.            
  result = alert->Go(); }                 // Get which button was pushed.
 if (result==1) {                         // Exit program choosen.
  fclose(PrinterPort);                    // Close the printer port.
  be_app->PostMessage(B_QUIT_REQUESTED);  // Sends a quit message to BApplication.
  return; } ;                             // Exit program, returns to BApplication.

 ref_num = 0; // Loop through the ref list and open each one
 do {
     err = message->FindRef("refs", ref_num, &ref);      // No more reference so exit.
     if (err != B_OK) { fclose(PrinterPort); return; } ; // If error setting file exit.
     err = FileObject.SetTo(&ref, B_READ_ONLY);          // Set BFile object to file to print.
     if (err != B_OK) { fclose(PrinterPort); return; } ; // If error setting file exit.
     FileObject.GetSize(&length);                        // Get size of file to print.
     FileLenght = FileObject.Read(FileBuffer, length);   // Read file to be printed into printer.  
     StartPrinting();                                    // Print out document.
     ref_num ++;                                         // Index to next reference.
    } 
    while (1);                                           // Loop continuously.
 }

/*-----------------------------Prototyping code needed -----------------------------------*/
 
int LineCount,ASC;
 status_t   err;        // The error/return code

void PrintLineBuffer() {
long index;
 err=fwrite(LineBuffer,1,LineLenght,PrinterPort); // err contains number of items written.
 for (index=0;index<LineLenght;index++) { LineBuffer[index]=(char)0; }; // Fill buffer with zeros.
 LineLenght=0; // Move pointer to start of buffer.
 return;
 }

void WriteOneByte() {
 LineBuffer[LineLenght]=(char)ASC; // Write one byte into buffer to send to printer.
 LineLenght++;                     // Add one(1) to lenght of buffer.
 return;
 }
 
void HexToBuffer(char* HexStr) {
long index;
int  ASC1,ASC2;
 index=0; 
 do {
  if ( HexStr[index]==(char)0 ) { return; };  // End of string. 
  if ( HexStr[index]==(char)32) { index++; }; // Space between hexadecimal codes.
  ASC1=(int)HexStr[index]; if (ASC1>95) {ASC1=ASC1-32;}; ASC1=ASC1-48; index++; if (ASC1>9) {ASC1=ASC1-7;} ; // First alphanumeric.
  ASC2=(int)HexStr[index]; if (ASC2>95) {ASC2=ASC2-32;}; ASC2=ASC2-48; index++; if (ASC2>9) {ASC2=ASC2-7;} ; // Second alphanumeric.
  ASC=ASC1*16+ASC2; WriteOneByte(); // Write one hexadecimal byte.
} while (1); }

void StartNewTextPage() {
long index;
 HexToBuffer(LoadPaper);                      // Load paper into printer. 
 if ( TopBlankLines>0 ) {                     // Print top margin in blank lins.
  for (index=0;index<TopBlankLines;index++) { // Count off blank lines.
   HexToBuffer(NewLine);                      // Print a new line.
  }//ENDFOR
 }//ENDIF
 LineCount=0;                                 // Clear number of text line printed on page.
 return;
}
 
void PrintTextPages() {            // Prints using printer's text mode.
long n;
 StartNewTextPage();               // Loads paper, 
 if (!(strcmp(FilterCodes,"YES"))) { goto FilterYes; }
 if (!(strcmp(FilterCodes,"Yes"))) { goto FilterYes; }
 if (!(strcmp(FilterCodes,"ALL"))) { goto FilterAll; }
 if (!(strcmp(FilterCodes,"All"))) { goto FilterAll; }

 // Default no filtering of any control codes
 for (n=0;n<FileLenght;n++) {      // Process entire file.
  if (LineCount==PaperY) {         // End of printable page?
   HexToBuffer(EjectPaper);        // If yes, eject paper and
   StartNewTextPage();             // Load a new sheet of paper.
  }//ENDIF
  ASC=(int)FileBuffer[n];          // Get a single character.
  WriteOneByte();                  // Print ASCII character.
 }//ENDFOR

FilterYes:                         // Filter end of line codes.
 for (n=0;n<FileLenght;n++) {      // Process entire file.
  if (LineCount==PaperY) {         // End of printable page?
   HexToBuffer(EjectPaper);        // If yes, eject paper and
   StartNewTextPage();             // Load a new sheet of paper.
  }//ENDIF
  ASC=(int)FileBuffer[n];          // Get a single character.
  switch(ASC) {                    // Test ASCII code for End of Line.
   case 10: {                      // Character is a LineFeed. 
    HexToBuffer(NewLine);          // Write new line code.
    PrintLineBuffer();             // Print line of text.
    LineCount++;                   // Increment line count.
    break; } 
   case 13: {                      // Character is a Carriage Return.
    HexToBuffer(NewLine);          // Write new line code.
    PrintLineBuffer();             // Print line of text.
    LineCount++;                   // Increment line count.
    break; }
    default: {                     // Not a LF or CR.
    WriteOneByte();                // Print ASCII character.
    break; }                      
  }//ENDSWITCH 
 }//ENDFOR

FilterAll:
 for (n=0;n<FileLenght;n++) {      // Process entire file.
  if (LineCount==PaperY) { // End of printable page?
   HexToBuffer(EjectPaper);        // If yes, eject paper and
   StartNewTextPage();             // Load a new sheet of paper.
  }//ENDIF
  ASC=(int)FileBuffer[n];          // Get a single character.
  switch(ASC) {                    // Test ASCII code for End of Line.
   case 9: {                       // Character is a Tab. 
    ASC=32;
    do {
     WriteOneByte();               // Write out one space.
     }//ENDDO
     while(LineLenght%TabSize);    // Loop until tab position reached.
    break; } 
   case 10: {                      // Character is a LineFeed. 
    HexToBuffer(NewLine);          // Write new line code.
    PrintLineBuffer();             // Print line of text.
    LineCount++;                   // Increment line count.
    break; } 
   case 13: {                      // Character is a Carriage Return.
    HexToBuffer(NewLine);          // Write new line code.
    PrintLineBuffer();             // Print line of text.
    LineCount++;                   // Increment line count.
    break; }
    default: {                     // Not a control character.
    if (ASC>31) WriteOneByte();    // Print ASCII character.
    break; }                      
  }//ENDSWITCH 
 }//ENDFOR

 PrintLineBuffer();                // Print line of text.
 HexToBuffer(EjectPaper);          // Eject paper and
 PrintLineBuffer();                // Print line of text.
 return;
}

void PrintGraphicPages() {         // Prints using printer's graphic mode.
 printf("Print Graphic Pages.\n"); // Give warning that graphic mode is not ready.
 alert = new BAlert("Printer Alert","Graphic Printing is not supported yet.","OK");                  
 result = alert->Go();             // Displays alert with warning.
 return;
} ;

void StartPrinting() {
 HexToBuffer(ResetPrinter);           // Reset printer to defaults.
 HexToBuffer(PrintCPI);               // Set characters per inch.
 HexToBuffer(PrintQuality);           // Set Draft, NLQ, LQ mode
 HexToBuffer(PrintSpacing);           // Set PS or non-PS mode
 HexToBuffer(PaperTray);              // Set PaperTray.
 HexToBuffer(PrintWrap);              // Set printer's linewrapping.
 HexToBuffer(PrintDirection);         // Set print directions.
 HexToBuffer(PrintOrientation);       // Set page Orientation.
 PrintLineBuffer();                   // Print line of text.
 if (strcmp(PrintOrient,"Portrait")) {// Test paper orientation.
  PaperX=(long)(CPI*LandscapeX);      // Width in characters.
  PaperY=(long)(LPI*LandscapeY); }    // Height in Lines.
 else {
  PaperX=(long)(CPI*PortraitX);       // Width in characters.
  PaperY=(long)(LPI*PortraitY); }     // Height in Lines.
 PaperY=PaperY-TopBlankLines;         // Skip bottom.
 if (strcmp(PrintMode,"Text")) {      // Test if to print in Text mode.
  PrintGraphicPages(); }              // If no, then print graphicly.
 else {
  PrintTextPages(); }                 // If yes, then print text.
 return;
} ;