// BePrinter 0.030 by Earl Colby Pottinger. 
// Partly based on code found in the Beos CDROM 
// Partly based on instructions found in "Programming the Be Operating System" By Dan Parks Sydow. 
// Additional help by Larry Baydak in debugging and coding/commenting style.

// Printer variables to control output.
char *PrinterPath= "/dev/parallel/parallel1"; // Printer's parallel port 1 interface as default.
char *PrintName="Epson Dot Matrix Compatible."; // Name of printer or printer family.
char *PrintMode="Text";    // Choose Text or Graphic mode.
char *FilterCodes="All";   // Filter out all control characters?
char *LineMode="WordWrap"; // How to handle long lines of text. (Truncate, LineWrap, WordWrap).

char *ResetPrinter="1B 40"; // Hexadecimal code for reseting printer, sent at start of first printing.
char *SelfTest="";          // Hexadecimal code for prints a self test print.
char *LoadPaper="";         // Hexadecimal code for loading paper before printing.
char *EjectPaper="0C";      // Hexadecimal code for ejecting present sheet of paper, may load another sheet.
char *CR_LF_FF="";          // Sets CR=CR  LF=LF  FF=FF.
char *NewLine="0A 0D";      // Start of next new line, Carriage Return and LineFeed.
char *Alarm="07 07 07";     // Sound alarm/buzzer.

 char *CPI10="12 1B 50"; // Hexadecimal code for 10 CPI. Uncompress, pica pitch.
float CPI_10=10.0;       // True CPI.
 char *CPI12="12 1B 4D"; // Hexadecimal code for 12 CPI. Uncompress, elite pitch.
float CPI_12=12.0;       // True CPI.
 char *CPI15="12 1B 67"; // Hexadecimal code for 15 CPI. Uncompress, micron pitch.
float CPI_15=15.0;       // True CPI.
 char *CPI17="0F 1B 50"; // Hexadecimal code for 17 CPI. Compress, pica pitch.
float CPI_17=17.0;       // True CPI.
 char *CPI20="0F 1B 4D"; // Hexadecimal code for 20 CPI. Compress, elite pitch.
float CPI_20=20.0;       // True CPI.

char *DraftMode="1B 78 00"; // Hexadecimal code for Draft Mode.
char *NLQ_Mode= "1B 78 01"; // Hexadecimal code for Near Letter Quality.
char *LQ_Mode=  "1B 78 02"; // Hexadecimal code for Letter Quality.

char *PS_ModeOn= "1B 70 01"; // Code for Proportional Spacing.
char *PS_ModeOff="1B 70 00"; // Code for Fixed Spacing.

 char *SelectExec="";      // Select Executive size paper. 7.25*10.5
float PortraitExecX=6.75;  // Width in inches.
float PortraitExecY=10.0;  // Height in inches.
float LandscapeExecX=10.0; // Width in inches.
float LandscapeExecY=6.75;  // Height in inches.

 char *SelectLetter="";      // Select Letter size paper. 8.5*11
float PortraitLetterX=8.0;   // Width in inches.
float PortraitLetterY=10.5;  // Height in inches.
float LandscapeLetterX=10.5; // Width in inches.
float LandscapeLetterY=8.0;  // Height in inches.

 char *SelectLegal="";      // Select Legal size paper. 8.5*14
float PortraitLegalX=8.0;   // Width in inches.
float PortraitLegalY=13.5;  // Height in inches.
float LandscapeLegalX=13.5; // Width in inches.
float LandscapeLegalY=8.0;  // Height in inches.

 char *SelectA4="";      // Select A4 size paper. 8.3*11.7
float PortraitA4X=7.8;  // Width in inches.
float PortraitA4Y=11.2;  // Height in inches.
float LandscapeA4X=11.2; // Width in inches.
float LandscapeA4Y=7.8; // Height in inches.

char *PortraitPage= ""; // Set to portrait orientation.
char *LandscapePage=""; // Set to landscape orientation.

char *EnableEOLWrap= ""; // Enable Printer's Linewrapping.
char *DisableEOLWrap=""; // Disable Printer's Linewrapping.
   
char *Unidirectional="1B 55 01"; // Print left to right only.
char *Bidirectional= "1B 55 00"; // Print in both directions.

long  TopBlankLines=3;    // Number of lines skip from Top of Page.
long  BottomBlankLines=3; // Number of lines skip from Top of Page.
                          // Printable lines of text on one page.
long  TabSize=4;          // Size of tab steps. 
long  PagesPerSheet=1;    // Number of pages to print on one sheet of paper.
long  ColumnsPerPage=1;   // Number of columns to print on one sheet of paper.

char *UnderlineOn= "1B 2D 01"; // Hexadecimal code for underlining text.
char *UnderlineOff="1B 2D 00"; // Hexadecimal code for turning underlining off.

char *BoldOn= "1B 47"; // Turn on Boldface.
char *BoldOff="1B 48"; // Turn off Boldface.

char *ItalicsOn= "1B 34"; // Hexadecimal code for italics on.
char *ItalicsOff="1B 35"; // Hexadecimal code for italics off.

char *Superscript= "1B 53 00"; // Turn on superscript.
char *Subscript=   "1B 53 01"; // Turn on subscript.
char *NormalScript="1B 54";    // Return to normal.

//char *Graphics="1B 4B";    // 0060 DPI.
//char *Graphics="1B 4C";    // 0120 DPI.
//char *Graphics="1B 59";    // 0120 DPI, High Speed.
//char *Graphics="1B 5A";    // 0240 DPI.
//char *Graphics="1B 2A 00"; // 0060 DPI.
//char *Graphics="1B 2A 01"; // 0120 DPI.
//char *Graphics="1B 2A 02"; // 0120 DPI, High Speed. 
//char *Graphics="1B 2A 03"; // 0240 DPI.
//char *Graphics="1B 2A 04"; // 0080 DPI.
//char *Graphics="1B 2A 06"; // 0090 DPI.
//char *Graphics="1B 2A 20"; // 0480 DPI.
//char *Graphics="1B 2A 21"; // 0960 DPI.
//char *Graphics="1B 2A 26"; // 0720 DPI.
//char *Graphics="1B 2A 27"; // 1440 DPI.
//char *Graphics="1B 2A 28"; // 2880 DPI.